/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.transports.http;

import java.io.ByteArrayOutputStream;
import java.io.StringWriter;
import java.lang.ref.WeakReference;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HeaderElement;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.xmlbeans.SchemaGlobalElement;
import org.apache.xmlbeans.SchemaType;
import org.apache.xmlbeans.SchemaTypeSystem;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlHexBinary;
import org.apache.xmlbeans.XmlObject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.submit.filters.WssRequestFilter;
import com.eviware.soapui.impl.wsdl.support.wss.IncomingWss;
import com.eviware.soapui.impl.wsdl.support.wss.WssContainer;
import com.eviware.soapui.impl.wsdl.support.xsd.SchemaUtils;
import com.eviware.soapui.model.iface.Attachment;
import com.eviware.soapui.model.propertyexpansion.PropertyExpansionContext;
import com.eviware.soapui.settings.HttpSettings;
import com.eviware.soapui.support.Tools;
import com.eviware.soapui.support.types.StringToStringMap;
import com.eviware.soapui.support.xml.XmlUtils;

/**
 * WsdlMockResponse for a MimeResponse
 * 
 * @author ole.matzura
 */

public class MimeMessageResponse implements WsdlResponse
{
	private final WeakReference<WsdlRequest> wsdlRequest;
	private long timeTaken;
	private long responseContentLength;
	private StringToStringMap requestHeaders;
	private StringToStringMap responseHeaders;
	private final String requestContent;
	private SSLInfo sslInfo;
	private MultipartMessageSupport mmSupport;
	private long timestamp;
	private Vector wssResult;
	private PostResponseDataSource postResponseDataSource;
	private byte[] requestData;

	public MimeMessageResponse(WsdlRequest wsdlRequest, final TimeablePostMethod postMethod, String requestContent, PropertyExpansionContext context)
	{
		this.wsdlRequest = new WeakReference<WsdlRequest>( wsdlRequest );
		this.requestContent = requestContent;
		this.timeTaken = postMethod.getTimeTaken();
		this.timestamp = System.currentTimeMillis();
		
		try
		{
			initHeaders( postMethod );
			sslInfo = postMethod.getSSLInfo();
			postResponseDataSource = new PostResponseDataSource( postMethod );
			responseContentLength = postResponseDataSource.getDataSize();
			
			Header h = postMethod.getResponseHeader( "Content-Type" );
			HeaderElement[] elements = h.getElements();
			
			String rootPartId = null;
			String multipartType = null;
			
			for( HeaderElement element : elements )
			{
				String name = element.getName().toUpperCase();
				if( name.startsWith( "MULTIPART/" ))
				{
					NameValuePair parameter = element.getParameterByName("start");
					if (parameter != null)
						rootPartId = parameter.getValue();
					
					parameter = element.getParameterByName("type");
					if (parameter != null)
						multipartType = parameter.getValue();
				}
			}
			
			mmSupport = new MultipartMessageSupport( postResponseDataSource, rootPartId, wsdlRequest.getOperation(), false,
						wsdlRequest.isPrettyPrint() );
			
			processIncomingWss( wsdlRequest, context );
			
			if (wsdlRequest.getSettings().getBoolean(HttpSettings.INCLUDE_RESPONSE_IN_TIME_TAKEN))
				this.timeTaken = postMethod.getTimeTakenUntilNow();
			
			if (wsdlRequest.isExpandMtomResponseAttachments() && "application/xop+xml".equals( multipartType ))
			{
				expandMtomAttachments( wsdlRequest );
			}
			
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			postMethod.getRequestEntity().writeRequest( out );
			requestData = out.toByteArray();
		}
		catch ( Exception e)
		{
			SoapUI.logError( e );
		}
	}

	private void processIncomingWss( WsdlRequest wsdlRequest, PropertyExpansionContext context )
	{
		IncomingWss incomingWss = ( IncomingWss ) context.getProperty( WssRequestFilter.INCOMING_WSS_PROPERTY );
		if( incomingWss != null )
		{
			try
			{
				Document document = XmlUtils.parseXml( mmSupport.getContentAsString() );
				wssResult = incomingWss.processIncoming( document, context );
				if( wssResult != null && wssResult.size() > 0 )
				{
					StringWriter writer = new StringWriter();
					XmlUtils.serializePretty( document, writer );
					mmSupport.setResponseContent( writer.toString() );
				}
			}
			catch( Exception e )
			{
				if( wssResult == null )
					wssResult = new Vector();
				wssResult.add( e );
			}
		}
	}
	
	private void expandMtomAttachments(WsdlRequest wsdlRequest)
	{
		try
		{
			XmlObject xmlObject = XmlObject.Factory.parse(getContentAsString());
			XmlObject[] includes = xmlObject.selectPath( "declare namespace xop='http://www.w3.org/2004/08/xop/include'; //xop:Include" );
			
			for( XmlObject include : includes )
			{
				Element elm = ( Element ) include.getDomNode();
				String href = elm.getAttribute( "href" );
				Attachment attachment = mmSupport.getAttachmentWithContentId( "<" + href.substring( 4 ) + ">" );
				if( attachment != null )
				{
					ByteArrayOutputStream data = Tools.readAll( attachment.getInputStream(), 0 );
					byte[] byteArray = data.toByteArray();
					
					XmlCursor cursor = include.newCursor();
					cursor.toParent();
					XmlObject parentXmlObject = cursor.getObject();
					cursor.dispose();
					
					SchemaType schemaType = parentXmlObject.schemaType();
					Node parentNode = elm.getParentNode();
					
					if( schemaType.isNoType())
					{
						SchemaTypeSystem typeSystem = wsdlRequest.getOperation().getInterface().getWsdlContext().getSchemaTypeSystem();
						SchemaGlobalElement schemaElement = typeSystem.findElement( new QName( parentNode.getNamespaceURI(), parentNode.getLocalName() ) );
						if( schemaElement != null )
						{
							schemaType = schemaElement.getType();
						}
					}

					String txt = null;
					
					if( SchemaUtils.isInstanceOf( schemaType, XmlHexBinary.type ))
					{
						txt = new String( Hex.encodeHex( byteArray ));
					}
					else
					{
						txt = new String( Base64.encodeBase64( byteArray ));
					}
					 
					
					parentNode.replaceChild( elm.getOwnerDocument().createTextNode( txt ), elm );
				}
			}
			
			mmSupport.setResponseContent( xmlObject.toString() );
		}
		catch( Exception e )
		{
			SoapUI.logError( e );
		}
	}

	public SSLInfo getSSLInfo()
	{
		return sslInfo;
	}

	public long getContentLength()
	{
		return responseContentLength;
	}

	public WsdlRequest getRequest()
	{
		return wsdlRequest.get();
	}

	public long getTimeTaken()
	{
		return timeTaken;
	}

	private void initHeaders(TimeablePostMethod postMethod)
	{
		requestHeaders = new StringToStringMap();
		Header[] headers = postMethod.getRequestHeaders();
		for( Header header : headers )
		{
			requestHeaders.put( header.getName(), header.getValue() );
		}
		
		responseHeaders = new StringToStringMap();
		headers = postMethod.getResponseHeaders();
		for( Header header : headers )
		{
			responseHeaders.put( header.getName(), header.getValue() );
		}
		
		responseHeaders.put( "#status#", postMethod.getStatusLine().toString() );
	}
	
	public StringToStringMap getRequestHeaders()
	{
		return requestHeaders;
	}

	public StringToStringMap getResponseHeaders()
	{
		return responseHeaders;
	}
	
	public String getRequestContent()
	{
		return requestContent;
	}

	public void setResponseContent( String responseContent )
	{
		String oldContent = getContentAsString();
		mmSupport.setResponseContent( responseContent );
		
		getRequest().notifyPropertyChanged( WsdlRequest.RESPONSE_CONTENT_PROPERTY, oldContent, responseContent );
	}

	public Attachment[] getAttachments()
	{
		return mmSupport.getAttachments();
	}

	public Attachment[] getAttachmentsForPart( String partName )
	{
		return mmSupport.getAttachmentsForPart( partName );
	}

	public String getContentAsString()
	{
		return mmSupport.getContentAsString();
	}

	public long getTimestamp()
	{
		return timestamp;
	}

	public Vector getWssResult()
	{
		return wssResult;
	}
	
	public byte[] getRawRequestData()
	{
		return requestData;
	}

	public byte[] getRawResponseData()
	{
		return postResponseDataSource.getData();
	}
}