/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.support.wsdl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.wsdl.Definition;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.SchemaType;
import org.apache.xmlbeans.SchemaTypeLoader;
import org.apache.xmlbeans.SchemaTypeSystem;
import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.DefinitionCacheConfig;
import com.eviware.soapui.config.DefintionPartConfig;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.support.soap.SoapVersion;
import com.eviware.soapui.impl.wsdl.support.xsd.SchemaException;
import com.eviware.soapui.impl.wsdl.support.xsd.SchemaUtils;
import com.eviware.soapui.support.NullProgressDialog;
import com.eviware.soapui.support.UISupport;
import com.eviware.x.dialogs.Worker;
import com.eviware.x.dialogs.XProgressDialog;
import com.eviware.x.dialogs.XProgressMonitor;

/**
 * Holder for WSDL4J Definitions and related SchemaTypeLoader types
 * 
 * @author Ole.Matzura
 */

public class WsdlContext 
{
   private String url;
   private Definition definition;
   private SchemaTypeLoader schemaTypes;
   private boolean loaded;
	private SchemaException schemaException;
	
	private final static Logger log = Logger.getLogger( WsdlContext.class );
	private SoapVersion soapVersion;
	private DefinitionCacheConfig cache;
	private WsdlLoader currentLoader;
	private WsdlInterface iface;
	private WSDLFactory factory;
	private WSDLReader wsdlReader;

   public WsdlContext( String url, SoapVersion soapVersion, DefinitionCacheConfig cache, WsdlInterface iface )
   {
   	this.url = url;
		this.soapVersion = soapVersion;
		this.cache = cache;
		this.iface = iface;
   }
   
   public WsdlInterface getInterface()
	{
		return iface;
	}

	public DefinitionCacheConfig getCacheConfig()
   {
   	return cache;
   }
   
   public Definition getDefinition() throws Exception
   {
      loadIfNecessary();
      return definition;
   }
   
   public Definition getDefinitionSilently() throws Exception
   {
      if( load(true) )
      {
         return getDefinition();
      }
      else
      {
         return null;
      }
   }
   
   public boolean isLoaded()
   {
   	return loaded;
   }
   
   public synchronized boolean loadIfNecessary() throws Exception
   {
   	return loadIfNecessary( false );
   }
   
   public synchronized boolean loadIfNecessary( boolean silent ) throws Exception
   {
   	if( !loaded )
         load( silent );
   	
   	return loaded;
   }
   
   public synchronized void setDefinition( String url, DefinitionCacheConfig cache ) 
   {
   	this.url = url;
		this.cache = null;
   	loaded = false;
   }
     
   public synchronized boolean load() throws Exception
   {
   	return load( null, false );
   }
   
   public synchronized boolean load(boolean silent) throws Exception
   {
   	return load( null, silent );
   }
   
   public synchronized boolean load( WsdlLoader wsdlLoader ) throws Exception
   {
   	return load( wsdlLoader, false );
   }
   
   public synchronized boolean load( WsdlLoader wsdlLoader, boolean silent ) throws Exception
   {
   	if( loaded )
   		return true;
   	
      XProgressDialog progressDialog;
      if( silent || url.startsWith("file:") )
      {
         progressDialog = new NullProgressDialog();
      }
      else
      {
         progressDialog = UISupport.getDialogs().createProgressDialog(
               "Loading WSDL", 3, "Loading definition..", true );
      }
      
   	Loader loader = new Loader( wsdlLoader );
      progressDialog.run( loader);
      
      // Get the value. It is the responsibility of the progressDialog to
      // wait for the other thread to finish.
      if( loader.hasError() )
      {
         if( loader.getError() instanceof SchemaException )
         {
         	schemaException = (SchemaException) loader.getError();
         	ArrayList errorList = schemaException.getErrorList();
         	
         	if( errorList != null )
         	{
	         	log.error( "Error loading schema types from " + url + ", see log for details" );
	         	for( int c = 0; c < errorList.size(); c++ )
	         	{
	         		log.error( errorList.get( c ).toString() );
	         	}
         	}
         	
         	if(!silent)
        			UISupport.showErrorMessage( "Error loading schema types from " + url + ", see log for details" );
         }
         else throw loader.getError();
      }
      else loaded = true;
      
      return loaded;
   }

   public SchemaTypeLoader getSchemaTypeLoader() throws Exception
   {
      loadIfNecessary();
      return schemaTypes;
   }
   
   public SchemaException getSchemaException()
   {
   	return schemaException;
   }

   private class Loader extends Worker.WorkerAdapter
   {
      private Exception error;
      private WsdlLoader wsdlLoader;

      public Loader(WsdlLoader wsdlLoader) 
      {
          super();
          this.wsdlLoader = wsdlLoader;
      }

      private WsdlLoader getWsdlLoader() 
      {
          if(wsdlLoader != null) {
              return wsdlLoader;
          } else {
              return new UrlWsdlLoader( url );
          }
      }
      
      public boolean hasError()
		{
			return error != null;
		}

		public Object construct(XProgressMonitor monitor)
      {
      	try
      	{
				if( !validateCache( cache ) )
				{
				   monitor.setProgress( 1, "Caching definition from url [" + url + "]" ); 
				   
				   currentLoader = getWsdlLoader();
				   cache = iface == null ? WsdlLoader.cacheWsdl( currentLoader ) : 
				   		iface.cacheDefinition( currentLoader );
				   
				   if( currentLoader.isAborted() )
				   	throw new Exception( "Loading of WSDL from [" + url + "] was aborted" );
				}
      		 
            monitor.setProgress( 1, "Loading definition from " + (cache == null ? "url" : "cache") );
            
            log.debug( "Loading definition from " + (cache == null ? "url" : "cache") );
            currentLoader = cache == null ? getWsdlLoader() : new CachedWsdlLoader( cache );
				loadDefinitions( currentLoader);
            return null;
         }
         catch (Exception e)
         {
         	log.error( "Loading of definition failed for [" + url + "]; " + e );
         	SoapUI.logError( e );
            this.error = e;
            return e;
         }
         finally
         {
            currentLoader = null;
         }
      }
      
      public Exception getError()
      {
         return error;
      }
      
   	public boolean onCancel()
   	{
   		if( currentLoader == null )
   			return false;

   		return currentLoader.abort();
   	}
   }

   private void loadDefinitions( WsdlLoader loader ) throws Exception
   {
   	currentLoader = loader;
   	
   	if( factory == null )
   	{
	      factory = WSDLFactory.newInstance();
			wsdlReader = factory.newWSDLReader();
			wsdlReader.setFeature("javax.wsdl.verbose", true);
	      wsdlReader.setFeature("javax.wsdl.importDocuments", true);
   	}

      definition = wsdlReader.readWSDL( loader );
      log.debug( "Loaded definition: " + (definition != null ? "ok" : "null") );

      if( !currentLoader.isAborted() )
      	schemaTypes = SchemaUtils.loadSchemaTypes(url, soapVersion, loader);
      else 
      	throw new Exception( "Loading of WSDL from [" + url + "] was aborted" );
   }
   
	public boolean validateCache(DefinitionCacheConfig cache)
	{
		if( cache == null )
			return false;
		
		if( cache.getRootPart() == null )
			return false;
		
		if( cache.sizeOfPartArray() == 0 )
			return false;
		
		return true;
	}

	public SchemaTypeSystem getSchemaTypeSystem() throws Exception
	{
      if( !loaded )
         load();
		
		if( schemaTypes == null )
			return null;
		
		return schemaTypes.findElement( soapVersion.getEnvelopeQName() ).getTypeSystem();
	}
   
   public boolean hasSchemaTypes() throws Exception
   {
      loadIfNecessary();
      return schemaTypes != null;
   }

   public SchemaType findType(QName typeName) throws Exception
   {
      loadIfNecessary();
      return schemaTypes == null ? null : schemaTypes.findType( typeName );
   }

   public String getUrl()
   {
      return url;
   }

	public Collection<String> getDefinedNamespaces() throws Exception
	{
      loadIfNecessary();
      Set<String> namespaces = new HashSet<String>();
      
      if( schemaTypes != null )
      {
      	namespaces.addAll( SchemaUtils.extractNamespaces( getSchemaTypeSystem(), true ));
      }
      
      if( definition != null )
      	namespaces.add( definition.getTargetNamespace() );
      
      return namespaces;
	}

	public SoapVersion getSoapVersion()
	{
		return soapVersion;
	}

	public void setSoapVersion(SoapVersion soapVersion)
	{
		this.soapVersion = soapVersion;
	}

	public Map<String, XmlObject> getDefinitionParts() throws Exception
	{
		Map<String,XmlObject> result = new HashMap<String,XmlObject>();
		
		if( cache == null )
			return SchemaUtils.getDefinitionParts( new UrlWsdlLoader( url ));
		
		List<DefintionPartConfig> partList = cache.getPartList();
		for( DefintionPartConfig part : partList )
		{
			result.put( part.getUrl(), CachedWsdlLoader.getPartContent( cache, part ) );
		}
		
		return result;
	}

	public void setDefinitionCache(DefinitionCacheConfig definitionCache)
	{
		this.cache = definitionCache;
	}

	public void setInterface( WsdlInterface iface )
	{
		this.iface = iface;
	}
}
