/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.x.form.support;

import java.lang.reflect.Field;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.swing.ActionList;
import com.eviware.x.form.XForm;
import com.eviware.x.form.XFormDialog;
import com.eviware.x.form.XFormDialogBuilder;
import com.eviware.x.form.XFormFactory;
import com.eviware.x.form.XFormField;
import com.eviware.x.form.XFormTextField;
import com.eviware.x.form.XForm.FieldType;

/**
 * Builds XFormDialogs from AForm/AField annotated classes/interfaces
 * 
 * @author ole.matzura
 */

public class ADialogBuilder
{
	public static XFormDialog buildDialog( Class<? extends Object> formClass )
	{
		return buildDialog( formClass, null );
	}

	public static XFormDialog buildDialog( Class<? extends Object> formClass, ActionList actions )
	{
		AForm formAnnotation = formClass.getAnnotation( AForm.class );
		if( formAnnotation == null )
		{
			throw new RuntimeException( "formClass is not annotated correctly.." );
		}
		
		XFormDialogBuilder builder = XFormFactory.createDialogBuilder(formAnnotation.name());
      XForm form = builder.createForm( "Basic" );
      
      for( Field field : formClass.getFields() )
      {
      	AField fieldAnnotation = field.getAnnotation( AField.class );
      	if( fieldAnnotation != null )
      	{
      		addFormField( form, fieldAnnotation );
      	}
      }
      
      ActionList defaultActions = formAnnotation.helpUrl() == null ? 
      			builder.buildOkCancelActions() : builder.buildOkCancelHelpActions( formAnnotation.helpUrl() );

      if( actions == null )
      	actions = defaultActions;
      else
      	actions.addActions( defaultActions );
      
      XFormDialog dialog = builder.buildDialog( actions, formAnnotation.description(), 
      			UISupport.createImageIcon( formAnnotation.icon() ));
      
		return dialog;
	}
	
	public static XFormDialog buildTabbedDialog( Class<? extends Object> tabbedFormClass, ActionList actions )
	{
		AForm formAnnotation = tabbedFormClass.getAnnotation( AForm.class );
		if( formAnnotation == null )
		{
			throw new RuntimeException( "formClass is not annotated correctly.." );
		}
		
		XFormDialogBuilder builder = XFormFactory.createDialogBuilder(formAnnotation.name());
		
	   for( Field field : tabbedFormClass.getFields() )
      {
	      APage pageAnnotation = field.getAnnotation( APage.class );
         if( pageAnnotation != null )
         {
            buildForm(builder, pageAnnotation.name(), field.getType());
         }
         
      	AField fieldAnnotation = field.getAnnotation( AField.class );
      	if( fieldAnnotation != null )
      	{
      		try
				{
					Class formClass = Class.forName( fieldAnnotation.description() );
					buildForm(builder, fieldAnnotation.name(), formClass);
				}
				catch( Exception e )
				{
					SoapUI.logError( e );
				}
      	}
      }
      
      ActionList defaultActions = formAnnotation.helpUrl().length() == 0 ? 
      			builder.buildOkCancelActions() : builder.buildOkCancelHelpActions( formAnnotation.helpUrl() );

      if( actions == null )
      	actions = defaultActions;
      else
      	actions.addActions( defaultActions );
      
      XFormDialog dialog = builder.buildDialog( actions, formAnnotation.description(), 
      			UISupport.createImageIcon( formAnnotation.icon() ));
      
		return dialog;
	}

   public static XFormDialog buildWizard( Class<? extends Object> tabbedFormClass )
   {
      AForm formAnnotation = tabbedFormClass.getAnnotation( AForm.class );
      if( formAnnotation == null )
      {
         throw new RuntimeException( "formClass is not annotated correctly.." );
      }
      
      XFormDialogBuilder builder = XFormFactory.createDialogBuilder(formAnnotation.name());
      
      for( Field field : tabbedFormClass.getFields() )
      {
         APage pageAnnotation = field.getAnnotation( APage.class );
         if( pageAnnotation != null )
         {
            buildForm(builder, pageAnnotation.name(), field.getType());
         }
      }
      
      XFormDialog dialog = builder.buildWizard( formAnnotation.description(), 
               UISupport.createImageIcon( formAnnotation.icon() ),
               formAnnotation.helpUrl() );
      
      return dialog;
   }

   private static void buildForm(XFormDialogBuilder builder, String name, Class formClass)
   {
      XForm form = builder.createForm( name );
      for( Field formField : formClass.getFields() )
      {
      	AField formFieldAnnotation = formField.getAnnotation( AField.class );
      	if( formFieldAnnotation != null )
      	{
      		addFormField( form, formFieldAnnotation );
      	}
      }
   }

	private static void addFormField( XForm form, AField fieldAnnotation )
	{
		XFormField field = null;
		
		switch( fieldAnnotation.type() )
		{
			case STRING :
				field = form.addTextField( fieldAnnotation.name(), fieldAnnotation.description(), FieldType.TEXT );
				break;
			case INT :
				field = form.addTextField( fieldAnnotation.name(), fieldAnnotation.description(), FieldType.TEXT );
				((XFormTextField)field).setWidth( 10 );
				break;
			case STRINGAREA :
				field = form.addTextField( fieldAnnotation.name(), fieldAnnotation.description(), FieldType.TEXTAREA );
				break;
			case BOOLEAN :
				field = form.addCheckBox( fieldAnnotation.name(), fieldAnnotation.description() );
				break;
			case FILE :
				field = form.addTextField( fieldAnnotation.name(), fieldAnnotation.description(), FieldType.FILE );
				break;
			case FOLDER :
				field = form.addTextField( fieldAnnotation.name(), fieldAnnotation.description(), FieldType.FOLDER );
				break;
			case ENUMERATION :
				field = form.addComboBox( fieldAnnotation.name(), fieldAnnotation.values(), fieldAnnotation.description() );
				break;
			case RADIOGROUP :
				field = form.addComponent( fieldAnnotation.name(), new XFormRadioGroup( fieldAnnotation.values() ) );
				break;
			case MULTILIST :
				field = form.addComponent( fieldAnnotation.name(), new XFormMultiSelectList( fieldAnnotation.values() ) );
				break;
			default :
				System.out.println( "Unsupported field type: " + fieldAnnotation.type() );
		}
		
		if( field != null )
			field.setEnabled( fieldAnnotation.enabled() );
	}

}
